#include "ks32c5000.h"
#include "flash.h"

extern void diag_printf(char *fmt, ...);
extern volatile unsigned char DallasDS1511W[];
extern void fis_init(void);
extern void fis_create(void);
unsigned long _cksum(unsigned long *buf, int len);

//------------------------------------------------------
void setupExternalAccess(void)
{
  EXTACON0 =  
    Tcos0(1)+Tacs0(0)+Tcoh0(1)+Tacc0(2)      // CPLD
   +Tcos1(2)+Tacs1(4)+Tcoh1(1)+Tacc1(1);     // AIOPICs

  EXTACON1 =
   +Tcos2(1)+Tacs2(0)+Tcoh2(4)+Tacc2(5)      // Dallas DS1511W
   +Tcos3(0)+Tacs3(0)+Tcoh3(0)+Tacc3(2);     // Ring Indicators
}

//--------------------------------------------------
#define DISABLE_INTERRUPTS()           \
    asm volatile (                     \
        "mrs r4,cpsr;"                 \
        "orr r4,r4,#0xC0;"             \
        "msr cpsr,r4"                  \
                    : \
                    : \
                    : "r4" \
        )

//------------------------------------------------------------
void done(void)
{
  extern volatile unsigned long PioMode;

  diag_printf("Resetting unit...\n\n");
  PioMode |= Bit(16);
  DallasDS1511W[0x0c] = 0x10;       // 0.10 second
  DallasDS1511W[0x0d] = 0x00;
  DallasDS1511W[0x0e] = 0x00;
  DallasDS1511W[0x0f] = 0x83;
  DISABLE_INTERRUPTS();
  while (1)
    ;
}

//---------------------------------------------------------------
#define NumElem(a) ((sizeof a)/(sizeof a[0]))

//---------------------------------------------------------
void kickWatchdog(void)
{
  DallasDS1511W[0x0c] = 0x00;
}

//------------------------------------------------------
int main(void)
{
//  extern unsigned short __payloadStart[];
//  extern unsigned short __payloadEnd[];
  int s;

  diag_printf("\nStarting Burn SocketServer Utility\n");

//  diag_printf("payload checksum=%08x\n", _cksum((unsigned long*)__payloadStart, 2*(__payloadEnd-__payloadStart)));
  
  setupExternalAccess();

  s = flash_hwr_init();
  if (s != FLASH_ERR_OK)
    {
      diag_printf("Flash query failed: %d\n",s);
      done();
    }
  
  fis_init(); 		//set up FIS directory table
  fis_create();		//load app and add an entry to the FIS table
  diag_printf("\nCompleted Burn SocketServer Utility - Sucessful\n\n");
  done();
  return 0;
}

//=========================================================================
