/*********************************************
 *
 *  hal_diag.c  -  Functions relating to writing 
 *                 debug UART.
 * 
 *********************************************/
#include "ks32c5000.h"
#include "iic.h"

extern tUart ks32c5000_uart1;
tS3C4510Uart *pUart4510;
tS3C4530Uart *pUart4530;

static int is4530;

void hal_diag_init(void);

void hal_diag_write_char(char c)
{
    unsigned int uartStatus;
  
    hal_diag_init();

    if (is4530) {
	do  {
	    uartStatus = pUart4530->ustat;
	} while (!(uartStatus &  S3C4530UartStatusTxHoldRegEmpty));
	pUart4530->utxbuf = c;
    }
    else {
	do  {
	    uartStatus = pUart4510->ustat;
	} while (!(uartStatus & UartStatusTxBufEmpty));
	pUart4510->utxbuf = c;
    }
}

void hal_diag_write_string(char *p)
  {
    while (*p)
      hal_diag_write_char(*p++);
  }

void hal_diag_read_char(char *c)
{
    unsigned int uartStatus;
    
    if (is4530) {
	do  {
	    uartStatus = pUart4530->ustat;
	} while (!(uartStatus & S3C4530UartStatusRxDataAvail));
	*c = pUart4530->urxbuf;
    }
    else {
	do  {
	    uartStatus = pUart4510->ustat;
	} while (!(uartStatus & UartStatusRxDataAvail));
	*c = pUart4510->urxbuf;
    }
}

void hal_diag_init(void)
{
  static int init;
  int is_dm500;
  unsigned long model_id, baud_clock, baud_divisor;
//  char *msg = "\n\rDiag Startup\n\r";

  if (init) 
    return;
  
  ++init;

  // set UART baud divisor based on model ID
  getModelNumber(&model_id);
  is_dm500 = (model_id == 5002362 || model_id == 5002363 || model_id == 5002117);
  baud_clock = (is_dm500 ? 48000000 : 44236800) / 2;
  baud_divisor = (((baud_clock / (16 * (57600))) - 1) << 4);

  //  Check the CPU product ID, 4510 vs. 4530 ARM processor
  is4530 = SYSCFG & Bit(27);
//  pUart4510 = (tS3C4510Uart *)&ks32c5000_uart1;
//  pUart4530 = (tS3C4530Uart *)&ks32c5000_uart1;
  pUart4510 = (void *)&ks32c5000_uart1;
  pUart4530 = (void *)&ks32c5000_uart1;


  // configure UART
  if (is4530) {
      pUart4530->ucon = S3C4530UartControlWordLen8 + 
	  S3C4530UartControlParityNone +
	  S3C4530UartControlTxModeInt +
	  S3C4530UartControlRxModeInt +
	  S3C4530UartControlTxFifoEn +
	  S3C4530UartControlRxFifoEn +
	  S3C4530UartControlDTR +
	  S3C4530UartControlRTS;
      pUart4530->brdiv = baud_divisor;
      
  }
  else {
      pUart4510->ulcon = UartLineWordLen8 + UartLineStop1 + UartLineParityNone;
      pUart4510->ucon  = UartControlRxModeInterrupt + UartControlTxModeInterrupt;
      pUart4510->brdiv = baud_divisor;
  }

//  while (*msg) 
//    hal_diag_write_char(*msg++);
}

/*---------------------------------------------------------------------------*/
/* End of hal_diag.c */
