/*
 * structure definitions for Samsung KS32C5000 peripheral registers
 */

typedef volatile unsigned int reg;

#define Bit(n) (1<<(n))


/*
 * Interrupt Controller
 */

typedef struct
{
  reg intmod;          // Interrupt mode register
  reg intpnd;          // Interrupt pending register
  reg intmsk;          // Interrupt mask register
  reg intpri0;         // Interrupt priority register
  reg intpri1;
  reg intpri2;
  reg intpri3;
  reg intpri4;
  reg intpri5;
  reg intoffset;       // Interrupt offset register
} tInterruptController;


/* The following bit masks are for use in the intmod, intpnd, and
 * intmsk registers
 */

#define IntMaskExt0     Bit(0)
#define IntMaskExt1     Bit(1)
#define IntMaskExt2     Bit(2)
#define IntMaskExt3     Bit(3)
#define IntMaskUart0Tx  Bit(4)
#define IntMaskUart0Rx  Bit(5)
#define IntMaskUart0Err Bit(6)
#define IntMaskUart1Tx  Bit(7)
#define IntMaskUart1Rx  Bit(8)
#define IntMaskUart1Err Bit(9)
#define IntMaskDma0     Bit(10)
#define IntMaskDma1     Bit(11)
#define IntMaskTimer0   Bit(12)
#define IntMaskTimer1   Bit(13)
#define IntMaskHDLCA    Bit(14)
#define IntMaskHDLCB    Bit(15)
#define IntMaskEtherBDMATx  Bit(16)
#define IntMaskEtherBDMARx  Bit(17)
#define IntMaskEtherMacTx   Bit(18)
#define IntMaskEtherMacRx   Bit(19)
#define IntMaskI2C      Bit(20)
#define IntMaskGlobal   Bit(21)



/*
 * Timers
 */

typedef struct
{
  reg tmod;      // Timer mode
  reg tdata0;    // Timer 0 Data
  reg tdata1;    // Timer 1 Data
  reg tcnt0;     // Timer 0 Count
  reg tcnt1;     // Timer 1 Count
}tTimers;

#define TimerModeEnable0  Bit(0)
#define TimerModeToggle0  Bit(1)
#define TimerModeInitOut0 Bit(2)
#define TimerModeEnable1  Bit(3)
#define TimerModeToggle1  Bit(4)
#define TimerModeInitOut1 Bit(5)


/*
 * UART
 */

typedef struct
{
  reg ulcon;   // UART Line Control
  reg ucon;    // UART Control
  reg ustat;   // UART Status
  reg utxbuf;  // UART Tx Buffer
  reg urxbuf;  // UART Rx Buffer
  reg brdiv;   // UART Baud Rate Divisor
  reg brdcnt;  // UART Baud Rate Counter
  reg brdclk;  // UART Baud Rate Clock
}tUart;

/* UART Line Control */

#define UartLineWordLenMask 0x03
#define UartLineWordLen5       0
#define UartLineWordLen6       1
#define UartLineWordLen7       2
#define UartLineWordLen8       3

#define UartLineStopMask    (1 << 2)
#define UartLineStop1       (0 << 2)
#define UartLineStop2       (1 << 2)

#define UartLineParityMask  (0x07 << 3)
#define UartLineParityNone   (0 << 3)
#define UartLineParityOdd    (4 << 3)
#define UartLineParityEven   (5 << 3)
#define UartLineParityMark   (6 << 3)
#define UartLineParitySpace  (7 << 3)

#define UartLineInfraRedEnable Bit(7)


/* UART Control */


#define UartControlRxModeMask      3
#define UartControlRxModeDisabled  0
#define UartControlRxModeInterrupt 1
#define UartControlRxModeDma0      2
#define UartControlRxModeDma1      3

#define UartControlRxStatusIntEnable Bit(2)

#define UartControlTxModeMask      (0x3 <<3)
#define UartControlTxModeDisable   (0 <<3)
#define UartControlTxModeInterrupt (1 <<3)
#define UartControlTxModeDma0      (2 <<3)
#define UartControlTxModeDma1      (3 <<3)

#define UartControlDSR             Bit(5)

#define UartControlSendBreak       Bit(6)

#define UartControlLoopback        Bit(7)


/* UART Status */

#define UartStatusRxOverrunError  Bit(0)
#define UartStatusRxParityError   Bit(1)
#define UartStatusRxFrameError    Bit(2)
#define UartStatusRxBreak         Bit(3)
#define UartStatusDTR             Bit(4)
#define UartStatusRxDataAvail     Bit(5)
#define UartStatusTxBufEmpty      Bit(6)
#define UartStatusTxDone          Bit(7)


#if 0

                BaudClock 
Baud rate = ------------------
            16 * (Divisor + 1)
    
Where divisor is bits 4-15 of the register value    

#endif

#define BaudClock (44236800L/2)
#define BaudDivisor(b) (((BaudClock/(16*(b)))-1)<<4)
    
// structure for external access control registers    
    
typedef struct
{
  unsigned empty1 :4;
  unsigned Tacc1  :3;
  unsigned Tcoh1  :3;
  unsigned Tacs1  :3;
  unsigned Tcos1  :3;

  unsigned empty0 :4;
  unsigned Tacc0  :3;
  unsigned Tcoh0  :3;
  unsigned Tacs0  :3;
  unsigned Tcos0  :3;

  unsigned empty3 :4;
  unsigned Tacc3  :3;
  unsigned Tcoh3  :3;
  unsigned Tacs3  :3;
  unsigned Tcos3  :3;
  
  unsigned empty2 :4;
  unsigned Tacc2  :3;
  unsigned Tcoh2  :3;
  unsigned Tacs2  :3;
  unsigned Tcos2  :3;
}tExtCon;


